//---------------------------------------------------------------------------
//
// Ray Credit Panel v1.0
//
// FREEWARE COMPONENT.
//---------------------------------------------------------------------------
// Ray Credit Panel v1.0
// Nice credit panel with some HTML support
// You can use HTML like Tags in text
//
// <b> -Turn on bold
// <i> -italic
// <u> -undeline
// </b> - Turn off bold
// </i>
// </u>
// <cRRGGBB> - set text color must be in HEX value size of 6 like FFAAE0
// </c> - reset to standard color
// <n[name of color]> - set named color
// available named colors:
(*
RED
GREEN
AQUA
BLACK
BLUE
DDKGRAY
FUCHSIA
GRAY
LIME
LTGRAY
MAROON
NAVY
OLIVE
PURPLE
SILVER
TEAL
WHITE
YELLOW
*)
// When you use TAGs, try to close it in single line
// like this:
// My <b>Software</b><cFF5566> Super System </c><b>v1.0</b>
//
// UseGradient - turn on using gradient while painting text from top to
// bottom
// BeginColor - Top color
// EndColor - bottom color
//---------------------------------------------------------------------------
//
//  Copyright  2001 Ray Adams
//  E-Mail: rayadams@writeme.com
//  WWW: http://rayadams.i.am
//  or
//  http://members.nbci.com/ray_adams
//
//                     IMPORTANT NOTE:
// This software is provided 'as-is', without any express or
// implied warranty. In no event will the author be held
// liable for any damages arising from the use of this
// software.
// Permission is granted to anyone to use this software for
// any purpose, including commercial applications, and to
// alter it and redistribute it freely, subject to the
// following restrictions:
// 1. The origin of this software must not be misrepresented,
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is
//    not required.
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original
//    software.
// 3. This notice may not be removed or altered from any
//    source distribution.
unit CRPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  typinfo,ExtCtrls;

type
  TGlyphDrawType=(dgStrech,dgTile);
  TCRPanel = class(TCustomPanel)
  private
    FStrings: TStrings;
    FTimer:TTimer;
    FActive: boolean;
    FUseGradient: boolean;
    FEndColor: TColor;
    FBeginColor: TColor;
    FGlyph: TBitmap;
    FDrawGlyph: boolean;
    FGlyphDrawType: TGlyphDrawType;
    procedure SetStrings(const Value: TStrings);
    procedure SetActive(const Value: boolean);
    procedure SetBeginColor(const Value: TColor);
    procedure SetEndColor(const Value: TColor);
    procedure SetUseGradient(const Value: boolean);
    procedure SetInterval(const Value: longint);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure WMSize(var Message: TMessage); message WM_SIZE;
    procedure SetGlyph(const Value: TBitmap);
    procedure SetDrawGlyph(const Value: boolean);
    procedure SetGlyphDrawType(const Value: TGlyphDrawType);
    function GetINterval: longint;
    { Private declarations }
  protected
    { Protected declarations }
    B:TBitmap;
    Th:longint;
    CurY,CurLine,LinesPerPage:longint;
    f_InColor:TColor;
    InTimer:boolean;
    CurAngle:longint;
    procedure Timer1Timer(Sender: TObject);
    function GetColorFade(BC,EC:TColor;steps,posittion:longint):TColor;
    procedure PreparePanel;
    procedure UnPreparePanel;
    procedure UpdateMEMImage;
    procedure Paint;override;
    procedure ListChange(Sender:TObject);
    procedure GlyphChanged(Sender: TObject);
    function NoTags(s:String):string;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy; override;
    property FinColor:TColor read F_Incolor write F_Incolor;
  published
    { Published declarations }
    property Align;
    property Alignment;
    property Anchors;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property Font;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property Strings:TStrings read FStrings write SetStrings;
    property Active:boolean read FActive write SetActive;
    property BeginColor:TColor read FBeginColor write SetBeginColor;
    property EndColor:TColor read FEndColor write SetEndColor;
    property UseGradient:boolean read FUseGradient write SetUseGradient;
    property Interval:longint read GetINterval write SetInterval;
    property Glyph:TBitmap read FGlyph write SetGlyph;
    property DrawGlyph:boolean read FDrawGlyph write SetDrawGlyph;
    property GlyphDrawType:TGlyphDrawType read FGlyphDrawType write SetGlyphDrawType;
    property Visible;
  end;

procedure Register;

implementation

procedure DrawWallpaper(DC: HDC; Bitmap: TBitmap; Recty: TRect; IsTile: Boolean;
                        Mode: Integer);
var
  X, Y: Integer;
begin
  with Recty, Bitmap do
    {Tiled or not}
    if not IsTile then
      BitBlt(DC, Left, Top, Right - Left, Bottom - Top, Canvas.Handle, Left, Top, Mode) else
      begin
        X := Left;
        Y := Top;
        while Y < Bottom do
        begin
          while X < Right do
          begin
            BitBlt(DC, X, Y, Right-X, Bottom-Y, Canvas.Handle, 0, 0, Mode);
            Inc(X, Width);
          end;
          X := Left;
          Inc(Y, Height);
        end;
      end;
end;


procedure Register;
begin
  RegisterComponents('Ray Adams', [TCRPanel]);
end;

{ TCRPanel }

procedure TCRPanel.CMColorChanged(var Message: TMessage);
begin
inherited;
PreparePanel;
end;

procedure TCRPanel.CMFontChanged(var Message: TMessage);
begin
inherited;
PreparePanel;
end;

constructor TCRPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Color:=clBlack;
  FStrings:=TStringList.Create;
  TStringList(FStrings).OnChange:=ListChange;
  FActive:=false;
  FTimer:=TTimer.Create(Self);
  FTimer.Enabled:=false;
  FTimer.OnTimer:=Timer1Timer;
  FTimer.Interval:=1;
  FGlyph:=TBitmap.Create;
  FGlyph.OnChange := GlyphChanged;

  fInColor:=Font.Color;
  FuseGradient:=false;
  FBeginColor:=StrToInt('$00FF7171');
  FEndColor:=StrToInt('$0080FFFF');
  PreparePanel;
  UpdateMEMImage;  
end;

destructor TCRPanel.Destroy;
begin
  FGlyph.free;
  FStrings.Free;
  FTimer.Free;
  UnPreparePanel;
  inherited;
end;

function TCRPanel.GetColorFade(BC, EC: TColor; steps,
  posittion: Integer): TColor;
var r1,r2,g1,g2,b1,b2:byte;
begin
r1:=GetRValue(BC);
r2:=GetRValue(EC);
r1:=r1+((r2 - r1) div steps * posittion);

g1:=GetgValue(BC);
g2:=GetgValue(EC);
g1:=g1+((g2 - g1) div steps * posittion);

b1:=GetBValue(BC);
b2:=GetBValue(EC);
b1:=b1+((b2 - b1) div steps * posittion);
result:=rgb(r1,g1,b1);
end;

function TCRPanel.GetINterval: longint;
begin
result:=FTimer.Interval;
end;

procedure TCRPanel.GlyphChanged(Sender: TObject);
begin
PreparePanel;
Paint;
end;

procedure TCRPanel.ListChange(Sender: TObject);
begin
//if Active then Active:=False;
PreparePanel;
end;

function TCRPanel.NoTags(s: String): string;
var st:String;
    outstr:string;
begin
 st:=s;
if pos('<',s)=0 then
   begin
   result:=s;
   exit;
   end;
 while pos('<',st)<>0 do
       begin
       outstr:=outstr+copy(st,1,pos('<',st)-1);
       delete(st,1,pos('<',st)+1);
       delete(st,1,pos('>',st));
       end;
result:=OutStr+st;       
end;

procedure TCRPanel.Paint;
label beg;
var j,ij:longint;
    ClPos:byte;
    fc,c:string;
    IsStyleUndef:boolean;
    UseFColor:boolean;
    inColor:TColor;
    s:String;
    UseTag:boolean;
begin
  if (Strings.Count = 0) or not Active then
     begin
       Canvas.Brush.Color:=Color;
       Canvas.Brush.Style:=bsSolid;
       Canvas.FillRect(ClientRect);
       exit;
     end;

  if FDrawGlyph then
     begin
       case FGlyphDrawType of
       dgStrech:b.Canvas.StretchDraw(ClientRect,FGlyph);
       dgTile:DrawWallPAper(B.Canvas.Handle,FGlyph,ClientRect,true,SRCCOPY);
       end;
     end
       else begin
            b.Canvas.Brush.Color:=Color;
            b.Canvas.Brush.Style:=bsSolid;
            b.Canvas.FillRect(ClientRect);
            end;

  for ij:=0 to LinesPerPage do
  begin
   if (CurLine+ij)<Strings.Count then
      begin
           b.Canvas.Brush.Style:=bsClear;
           ClPos:=CurY div th;
           s:=Strings[CurLine+ij];
           case Alignment of
           taCenter:begin

                         j:=B.Canvas.TextWidth(NoTags(s)) div 2;
                         j:=(b.Width div 2) - j;
                    end;
           taRightJustify:j:=b.Width-B.Canvas.TextWidth(NoTags(s));
           else j:=0;
           end;
           UseFColor:=false;
           UseTag:=false;
           while Length(s)>0 do
           begin
           IsStyleUndef:=false;
           if (pos('<',s)<>0) and (s[1]='<') and not UseTag then
              begin
                  UseTag:=false;
                  delete(s,1,1);
                  beg:
                  c:=s[1];
                  case AnsiUpperCase(c)[1] of
                  'N':begin
                           //predefined colors
                           if not IsStyleUndef then
                              begin
                              delete(s,1,1);
                              fc:=aNSIuPPERcASE(copy(s,1,pos('>',s)-1));
                              delete(s,1,pos('>',s));
                              UseFColor:=true;
                                if fc='RED' then FInColor:=clRed
                                else
                                if fc='GREEN' then FInColor:=clGreen
                                else
                                if fc='AQUA' then FIncOLor:=clAqua
                                else
                                if fc='BLACK' then FIncOLor:=clBlack
                                else
                                if fc='BLUE' then FIncOLor:=clBlue
                                else
                                if fc='DDKGRAY' then FIncOLor:=clDkGray
                                else
                                if fc='FUCHSIA' then FIncOLor:=clFuchsia
                                else
                                if fc='GRAY' then FIncOLor:=clGray
                                else
                                if fc='LIME' then FIncOLor:=clLime
                                else
                                if fc='LTGRAY' then FIncOLor:=clLtGray
                                else
                                if fc='MAROON' then FIncOLor:=clMaroon
                                else
                                if fc='NAVY' then FIncOLor:=clNavy
                                else
                                if fc='OLIVE' then FIncOLor:=clOlive
                                else
                                if fc='PURPLE' then FIncOLor:=clPurple
                                else
                                if fc='SILVER' then FIncOLor:=clSilver
                                else
                                if fc='TEAL' then FIncOLor:=clTeal
                                else
                                if fc='WHITE' then FIncOLor:=clWhite
                                else
                                if fc='YELLOW' then FIncOLor:=clYellow
                                else

                                ;

                              end else
                              begin
                                   FInColor:=Font.Color;
                                   UseFColor:=false;
                                   delete(s,1,2);
                              end;
                           end;
                  'C':begin
                           //color definition

                           if not IsStyleUndef then
                           begin
                                UseFColor:=true;
                                delete(s,1,1);
                                fc:=copy(s,1,6);
                                delete(s,1,5);
                                FInColor:=StrToInt('$00'+fc);
                           end else
                               begin
                               FInColor:=Font.Color;
                               UseFColor:=false;
                               end;
                           delete(s,1,2);
                           end;
                  'B':begin
                         if not IsStyleUndef then
                         b.Canvas.Font.Style:=b.Canvas.Font.Style+[fsBold]
                         else
                         b.Canvas.Font.Style:=b.Canvas.Font.Style-[fsBold];
                         delete(s,1,2);
                         end;
                  'I':begin
                         if not IsStyLeUndef then
                         b.Canvas.Font.Style:=b.Canvas.Font.Style+[fsItalic]
                         else
                         b.Canvas.Font.Style:=b.Canvas.Font.Style-[fsItalic];
                         delete(s,1,2);
                         end;
                  'U':begin
                      if not IsStyleUndef then
                         b.Canvas.Font.Style:=b.Canvas.Font.Style+[fsUnderline]
                         else
                         b.Canvas.Font.Style:=b.Canvas.Font.Style-[fsUnderline];
                      delete(s,1,2);
                      end;
                  '/':begin
                           delete(s,1,1);
                           IsStyleUndef:=true;

                           Goto beg;
                       end;//begin
                  '<':begin
                           delete(s,1,1);
                           c:='<';
                           UseTag:=true;
                           end;
                  end;//case
                  if not UseTag then continue;

              end else
               begin
                if pos('<',s)=0 then
                   begin
                        c:=s;s:='';
                   end else
                   begin
                        c:=copy(s,1,pos('<',s)-1);
                        delete(s,1,pos('<',s)-1);
                        UseTag:=false;
                   end;
               end;


           if FuseGradient then
           begin
                inColor:=GetColorFade(FBeginColor,FEndColor,LinesPerPage,ClPos+ij);
           end else inColor:=finColor;
           if UseFColor then inColor:=finColor;
            b.Canvas.Font.Color:=inColor;
            b.Canvas.TextOut(j,CurY+ij*th,c);
            j:=j+b.Canvas.TextWidth(c);
           end;

      end else break;
  end;
  BitBlt(Canvas.Handle,0,0,Width,Height,b.Canvas.Handle,0,0,SRCCOPY);
  dec(CurY);
  if CurY<(th*-1) then
   begin
             inc(CurLine);CurY:=0;
             if CurLine>=Strings.Count then
                begin
                CurLine:=0;
                CurY:=B.Height;
                end;
   end;
end;

procedure TCRPanel.PreparePanel;
begin
  if b=nil then b:=TBitmap.Create;
  b.Canvas.Font.Assign(Font);
  b.Canvas.Brush.color:=Color;
  th:=b.Canvas.TextHeight('W');
  LinesPerPage:=b.Height div (th+3)+5;
  CurLine:=0;
  CurY:=b.Height;
  FinColor:=Font.Color;
//  FTimer.Interval:=FInterval;
end;

procedure TCRPanel.SetActive(const Value: boolean);
begin
   FActive := Value;
   UnPreparePanel;
   PreparePanel;
//   FTimer.Interval:=Value;
   FTimer.Enabled:=FActive;
   Paint;
end;

procedure TCRPanel.SetBeginColor(const Value: TColor);
begin
  FBeginColor := Value;
end;

procedure TCRPanel.SetDrawGlyph(const Value: boolean);
begin
  FDrawGlyph := Value;
  if not FActive then Paint;
end;

procedure TCRPanel.SetEndColor(const Value: TColor);
begin
  FEndColor := Value;
end;

procedure TCRPanel.SetGlyph(const Value: TBitmap);
begin
  FGlyph.Assign(Value);
  if not FActive then Paint;
end;

procedure TCRPanel.SetGlyphDrawType(const Value: TGlyphDrawType);
begin
  FGlyphDrawType := Value;
  if not FActive then Paint;
end;

procedure TCRPanel.SetInterval(const Value: longint);
begin
  FTimer.Interval:=Value;
end;

procedure TCRPanel.SetStrings(const Value: TStrings);
begin
  FStrings.Assign(Value);
  PreparePanel;
  Paint;
end;

procedure TCRPanel.SetUseGradient(const Value: boolean);
begin
  FUseGradient := Value;
end;

procedure TCRPanel.Timer1Timer(Sender: TObject);
begin
if InTimer then exit;
InTimer:=true;
paint;
InTimer:=false;
end;

procedure TCRPanel.UnPreparePanel;
begin
{}
end;


procedure TCRPanel.UpdateMEMImage;
begin
  b.Width:=Width;
  b.Height:=Height;
  b.PixelFormat:=pfDevice;
end;

procedure TCRPanel.WMSize(var Message: TMessage);
begin
inherited;
while inTimer do ;
UpdateMEMImage;
PreparePanel;
Paint;
end;

end.
